/********************************************************
 * page -- Print selected pages from a Postscript file	*
 *							*
 * Usage:						*
 *	page <file> <page-list>				*
 *							*
 * <page-list> is of the form				*
 *	<page> or <page>-<page>				*
 *							*
 * Pages are represented by the actual number of the 	*
 * page on the slide.					*
 *							*
 * Note: This was designed to work with the postscript	*
 * slides for "The Teacher's Guide to			*
 * Practical C++ Programing"  Although it probably will	*
 * work on other postscript output, it has not been	*
 * generally tested.					*
 ********************************************************/
#include <fstream.h>
#include <string.h>
#include <stdio.h>

static ifstream in_file;	// File we are reading
static char line[4096];		// Line we are reading
static int setup_count = 0;	// Number of "%%BeginSetup" 
				// lines we've seen

/********************************************************
 * Begins -- Returns true if the current line begins	*
 *	with the give string.				*
 ********************************************************/
static inline bool begins(
   const char str[]	// String to check in the line
)
{
   return (strncmp(line, str, strlen(str)) == 0);
}
/********************************************************
 * out_line -- send the current line to the stdout	*
 ********************************************************/
static void out_line(void)
{
    cout << line << '\n';
    cout.flush();
}
/********************************************************
 * next_line -- read the next line.  If we hit the 	*
 *	trailer, write out the rest of the page and 	*
 *	exit.						*
 ********************************************************/
static void next_line(void)
{
    line[0] = '\0';
    in_file.getline(line, sizeof(line));

    if (begins("%%BeginSetup"))
	++setup_count;

    if (begins("%%Trailer"))
	setup_count--;

    if (begins("%%Trailer") && (setup_count == 0)) {
	while (!in_file.eof()) {
	   out_line();
	   line[0] = '\0';
	   in_file.getline(line, sizeof(line));
	}
	exit (0);
    }
}
/********************************************************
 * copy_until -- Copy until we hit a keys string	*
 ********************************************************/
static void copy_until(
   const char key_str[],	// String to look for
   const bool copy_flag		// If set, copy skipped text
)
{
    while (1) {
	// Move up a line
	next_line();

	// Copy line 
	if (copy_flag)
	    out_line();

	// Is this the line we want
	if (begins(key_str)) 
	    break;
    }
}
/********************************************************
 * read_until -- Read until we spot a page 		*
 ********************************************************/
static void read_until(
   const int page,		// Page to find
   const bool copy_flag		// If set, write pages to stdout
)
{
    char page_str[100];	// String to look for at the page start

    // Convert the page into a string to look for
    sprintf(page_str, "%%%%Page: \"%d\"", page);
    copy_until(page_str, copy_flag);
}



main(int argc, char *argv[])
{
    if (argc < 2) {
	cerr << "Usage is " << argv[0] << " <file> <pages>\n";
	exit (8);
    }
   in_file.open(argv[1]);		// Open the input file

    if (in_file.bad()) {
	cerr << "Error: Unable to open " << argv[1] << '\n';
	exit (8);
    }

   copy_until("%%EndSetup", true);

   while (argc > 2) {
       if (strchr(argv[2], '-') == NULL) {
	  // Single page number
	  read_until(atoi(argv[2]), false);
          out_line();
	  copy_until("%%EndPage", true);
	} else {
	   int start, end;	// Starting, ending page numbers

	   sscanf(argv[2], "%d-%d", &start, &end);
	   read_until(start, false);
	   out_line();

	   read_until(end, true);
	   copy_until("%%EndPage", true);
	}
	argv++;
	argc--;
    }
    copy_until("%%EOF_EOF", false);
    return (0);
}

